/*
    S60Bible A Bible Reader for SymbianOS
    Copyright (C) 2004 Yohanes Nugroho

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Yohanes Nugroho (yohanes@opensource.or.id)
    Kp Areman RT 09/08 No 71 Desa Tugu Cimanggis Depok 16951 Indonesia
*/


/*
 * Filename: Database.cpp
 * Description: Database Caching Class
 * Author: Yohanes Nugroho
 * Creation Date:
 * Last Update: 15/Feb/2004
 * Revisions:
 */
#ifdef USE_LIB_C
#include <stdio.h>
#include <stdlib.h>
#include "usermem.h"
#else
#include <e32std.h>
#include <w32std.h>
#endif

#include "const.h"
#include "PDBAccess.h"
#include "Database.h"


/**
 * Default constructor
 */
EXPORT_C Database::Database(PDBAccess *access)
{
	rec_count = 0;
	record = 0;
	pdbaccess = access;
}

/**
 * Destructor, frees up memory user by record
 */
EXPORT_C Database::~Database()
{
	DmxFreeAll();
}

static inline int compare(record_handle *r, int v, int no)
{
  if (r->version!=v) return (r->version-v);
  return r->recno-no;
}


/**
 * Add database record to the cache
 */
void* Database::add_record_handle(int version, int recno)
{
  /*
	for(int i = 0; i<rec_count; i++){
		if (record[i].version==version && record[i].recno==recno) {
			User::Free(record[i].memory);
			record[i].memory = memory;
			return;
		}
		}*/

	record = (record_handle *)User::ReAlloc(record,
			sizeof(record_handle)*(rec_count+1));
	if (!record) return NULL;

	void *memory = (void *)(pdbaccess->readRecord(recno));

	int pos = 0;
	if (rec_count>0) {
	  int i;
	  for (i = 0; i<rec_count; i++){
	    if (compare(&record[i], version, recno)>0) {
	      break;
	    }
	  }
	  if (i<rec_count) {
	    for(int j=rec_count; j>i; j--){
	      record[j] = record[j-1];
	    }
	  }
	  pos = i;
	}

	if (record) {
		record[pos].version = version;
		record[pos].recno = recno;
		record[pos].memory = memory;
		rec_count++;
	}
	return memory;
}

/**
 * Remove database record from the cache
 */
record_handle Database::remove_record(int version, int recno)
{
	record_handle rh;
	rh.version = -1;

	for(int i = 0; i<rec_count; i++){
                if (record[i].version==version && record[i].recno==recno) {
			rh = record[i];
			for(int j=i; j<rec_count-1; j++){
				record[j] = record[j+1];
			}
			rec_count--;
			return rh;
		}
	}

	return rh;
}

/**
 * Access a record
 * @param version bible version
 * @param recno record number
 * @returns pointer to the record
 */
void* Database::DmxLockVersionRecord(UInt16 version, UInt16 recno)
{
	int base=0;
	int pos = 0;
	int lim;
	for (lim = rec_count; lim != 0; lim >>= 1) {
	  pos = base + (lim >> 1);
	  int cmp = compare(&record[pos], version, recno);
	  if (cmp==0) {
	    return record[pos].memory;
	  }
	  if (cmp<0) {
	    base = pos + 1;
	    lim--;
	  }
	}
	void *mem = add_record_handle(version, recno);
	return mem;
}

/**
 * Release record from cache
 */
void Database::DmxUnlockVersionRecord(UInt16 version, UInt16 recno)
{
	record_handle rh;
	rh = remove_record(version, recno);
	if (rh.version!=-1) {
		User::Free(rh.memory);
	}
}

/**
 * Clear all cache entries
 */
void Database::DmxFreeAll()
{
	for(int i = 0; i<rec_count; i++){
		User::Free(record[i].memory);
	}
	User::Free(record);
}


/**
 * Retrieve number of versions
 */
UInt16 Database::DmxGetTotalVersions()
{
	return 1;
}

