/*
    S60Bible A Bible Reader for SymbianOS
    Copyright (C) 2004 Yohanes Nugroho

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Yohanes Nugroho (yohanes@opensource.or.id)
    Kp Areman RT 09/08 No 71 Desa Tugu Cimanggis Depok 16951 Indonesia
*/


#include "BibleSettingListView.h"
#include <S60Bible.rsg>
#include "S60Bible.hrh"
#include "BibleSettingList.h"
#include <barsread.h>
#include "CStringList.h"
#include "S60BibleUid.h"
#include "S60BibleDocument.h"


// factory constructor leaving instance on cleanup
CBibleSettingListView* CBibleSettingListView::NewLC(const TRect& aRect, CS60BibleDocument* aDocument)
{
	CBibleSettingListView* self = new (ELeave) CBibleSettingListView();
	CleanupStack::PushL(self);
	self->ConstructL(aRect, aDocument);
	return self;
}

// factory constructor - no instance left on cleanup
CBibleSettingListView* CBibleSettingListView::NewL(const TRect& aRect, CS60BibleDocument* aDocument)
{
	CBibleSettingListView* self = CBibleSettingListView::NewLC(aRect, aDocument);
	CleanupStack::Pop(self);
	return self;
}

// second phase constructor
void CBibleSettingListView::ConstructL(const TRect& /*aRect*/,  CS60BibleDocument* aDocument)
{
	// construct from resource
	BaseConstructL(R_FIRSTVIEW);
	iDocument = aDocument;
}

// first phase (non-leaving) constructor
CBibleSettingListView::CBibleSettingListView()
: CAknView()
{
	
}

void CBibleSettingListView::LoadSettings()
{
	
	CStringList *settings = CStringList::NewLC(CEikonEnv::Static()->FsSession());
	settings->LoadFromFileL(iDocument->Model()->iOptionsFile);
	if (settings->Count()>1){
		iSettingList->iFullScreen = settings->GetNumberAt(0);
		iSettingList->iBackLight = settings->GetNumberAt(1);
	}
	if (settings->Count()>2){
		iSettingList->iFullBookName = settings->GetNumberAt(2);
	}	
	
	CleanupStack::PopAndDestroy(settings);
}


// activation of view
void CBibleSettingListView::DoActivateL(const TVwsViewId& aPrevViewId, 
										 TUid /*aCustomMessageId*/, 
								   const TDesC8& /*aCustomMessage*/)
{
	iPrevView = aPrevViewId;
	if(!iSettingList)	// if we haven't already created iSettingList
		{
		// construct control and set parent
		iSettingList = CBibleSettingList::NewL(this);
		LoadSettings();
		iSettingList->SetMopParent(this);

		// construct control based on resource definition
		TResourceReader reader;

		// CreateResourceReaderLC will allocate a buffer to be used by
		// the TResourceReader. This buffer is pushed onto the cleanup
		// stack - not the TResourceReader itself
		CEikonEnv::Static()->CreateResourceReaderLC(reader, R_SETTINGS);
		iSettingList->ConstructFromResourceL(reader);

		// Clean up the buffer allocated above, NOT the reader itself.
		// Cannot use expected item overload of PopAndDestroy() as buffer 
		// is inaccessible. 
		CleanupStack::PopAndDestroy();	

		// register view and activate
		AppUi()->AddToViewStackL(*this, iSettingList);
		iSettingList->ActivateL();
		}
}

// deactivation of view
void CBibleSettingListView::DoDeactivate()
{
	if(iSettingList)	// if setting list has been created
		{
		// remove setting list from stack
		AppUi()->RemoveFromViewStack(*this, iSettingList);

		// clean up 
		delete iSettingList;
		iSettingList = NULL;
		}
}

TUid CBibleSettingListView::Id() const
{
	return TUid::Uid(KSettingListViewID);
}

void CBibleSettingListView::HandleCommandL(TInt aCommand)
{
	switch(aCommand)
		{
	case ESettingCmdEdit :					// edit current item
		iSettingList->EditCurrentItemL();
		break;
	case EAknSoftkeyBack:
		AppUi()->ActivateLocalViewL(iPrevView.iViewUid);
		break;
	case KSaveOptions:
		//Save
		{
			iSettingList->StoreSettingsL();
			CStringList *settings = CStringList::NewLC(CEikonEnv::Static()->FsSession());
			settings->AddNumberL(iSettingList->iFullScreen);
			settings->AddNumberL(iSettingList->iBackLight);
			settings->AddNumberL(iSettingList->iFullBookName);			
			settings->SaveToFileL(iDocument->Model()->iOptionsFile);
			CleanupStack::PopAndDestroy(settings);
			AppUi()->ActivateLocalViewL(iPrevView.iViewUid);
			break;
		}
	default :								// anything else
		AppUi()->HandleCommandL(aCommand);	// handled by AppUi
		break;
		}
}


void CBibleSettingListView::SizeChanged()
{
	if (iSettingList) {
		iSettingList->SetRect(ClientRect());
	}
}
